
import os
from os.path import expanduser
import logging
from logging.handlers import RotatingFileHandler
from msi_auth import MSIAuth
from aad_auth import AADAuth


# Specify files accessed by this script in guestshell
base_dir = expanduser('~') + '/cloud/authMgr/'
debug_file = base_dir + "token_svr.log"
aad_app_file = base_dir + "aad_app_file"

if not os.path.exists(base_dir):
    os.makedirs(base_dir)

default_msi_app = None
default_aad_app = None
aad_app_list = []


def find_aad_app(tenant_id, app_id, app_key):
    global aad_app_list
    for app in aad_app_list:
        if (app.app_id == app_id and
            app.tenant_id == tenant_id and
            app.app_key == app_key):
            # Found a match. Return this app.
            return app
    return None


def set_default_msi_app(logger):
    global default_msi_app
    msi_app = MSIAuth(logger)
    if msi_app is not None:
        default_msi_app = msi_app
        # Check if we got a token
        if msi_app.token == '':
            return 'ERR: Failed to obtain token using MSI application'
        else:
            return 'OK'
    else:
        return 'ERR: Failed to set default MSI application'


def set_default_aad_app(logger, cloud, tenant_id, app_id, app_key):
    global default_aad_app
    aad_app = AADAuth(logger, cloud, tenant_id, app_id, app_key)
    if aad_app is not None:
        default_aad_app = aad_app
        write_table_to_file()
        # Check if we got a token
        if aad_app.token == '':
            return 'ERR: Failed to obtain token for default AAD application'
        else:
            return 'OK'
    else:
        return 'ERR: Failed to set default AAD application'


def clear_default_aad_app():
    global default_aad_app
    default_aad_app = None
    if os.path.exists(aad_app_file):
        cmd = "rm %s" % aad_app_file
        os.system(cmd)
    return 'OK'


def clear_aad_app_list():
    global aad_app_list
    aad_app_list = []
    return 'OK'


def verify_aad_app(logger, cloud, tenant_id, app_id, app_key):
    # Try to find this application on the existing AAD app list
    app = find_aad_app(tenant_id, app_id, app_key)
    if app is not None:
        # Application is verified. Return the app
        return app

    # This is a new application.  Try to get a token using it.
    app = AADAuth(logger, cloud, tenant_id, app_id, app_key)
    token = app.get_token()
    if token != '':
        # New application is verified. Add it to the list.
        aad_app_list.append(app)
        return app
    else:
        # Get of the token failed
        logger.error("AAD application with app_id %s failed to obtain a token" % app_id)
        return None

def get_token(logger, app=None):
    global aad_app_list
    if app is not None:
        # Get the token using the specified application
        token = app.get_token()
        if token == '':
            # Check if this app is on the verified list and remove it
            for entry in aad_app_list:
                if (entry.cloud == app.cloud and
                    entry.app_id == app.app_id and
                    entry.tenant_id == app.tenant_id and
                    entry.app_key == app.app_key):
                    logger.error("AAD application with app_id %s has been invalidated" % app.app_id)
                    aad_app_list.remove(entry)
                    break
        return token

    # If no app was specified, try the default AAD app
    if default_aad_app is not None:
        token = default_aad_app.get_token()
        return token

    # If all else fails, try the default MSI app
    if default_msi_app is not None:
        token = default_msi_app.get_token()
        return token

    logger.error("No authentication application found for getting a token")
    return ''

def refresh_token(logger, app=None):
    global aad_app_list
    if app is not None:
        return app.refresh_token()

    # If no app was specified, try the default AAD app
    if default_aad_app is not None:
        return default_aad_app.refresh_token()

    # If all else fails, try the default MSI app
    if default_msi_app is not None:
        return default_msi_app.refresh_token()

    logger.error("No authentication application found for refreshing a token")
    return 'ERR: No authentication application found for refreshing a token'

def clear_token(logger, app=None):
    global aad_app_list
    if app is not None:
        return app.clear_token()

    # If no app was specified, try the default AAD app
    if default_aad_app is not None:
        return default_aad_app.clear_token()

    # If all else fails, try the default MSI app
    if default_msi_app is not None:
        return default_msi_app.clear_token()

    logger.error("No authentication application found for clearing a token")
    return 'ERR: No authentication application found for clearing a token'

def write_table_to_file():
    with open(aad_app_file, 'w') as write_fh:
        app_desc = "{cloud: %s, app_id: %s, tenant_id: %s, app_key: %s}" % (default_aad_app.cloud, default_aad_app.app_id, default_aad_app.tenant_id, default_aad_app.app_key)
        out_str = str(app_desc) + '\n'
        print "Writing app_desc to file"
        print "%s" % out_str
        write_fh.write(out_str)

def read_table_from_file(logger):
    if os.path.exists(aad_app_file):
        with open(aad_app_file, 'r') as read_fh:
            for line in read_fh:
                input_str = line.strip()
                print "Read table from file"
                print "%s" % input_str
                substr1 = input_str.replace('{', '')
                substr2 = substr1.replace('}', '')
                # Separate items into list of strings
                list_of_pairs = substr2.rsplit(',')
                # Create an empty dict
                out_var={}
                for list_entry in list_of_pairs:
                    # Split the entry into a key and value
                    key_value_pair = list_entry.rsplit(':')
                    key = key_value_pair[0].strip()
                    value = key_value_pair[1].strip()
                    out_var[key] = value
                set_default_aad_app(logger, out_var['cloud'], out_var['app_id'], out_var['tenant_id'], out_var['app_key'])


def show():
    global default_msi_app, default_aad_app, aad_app_list
    buf = ''
    if default_msi_app is not None:
        buf = buf + "\nDefault MSI application:"
        buf = buf + default_msi_app.show()
    else:
        buf = buf + "\nNo default MSI application is configured"

    if default_aad_app is not None:
        buf = buf + "\nDefault AAD application:"
        buf = buf + default_aad_app.show()
    else:
        buf = buf + "\nNo default AAD application is configured"

    if len(aad_app_list) > 0:
        buf = buf + "\nListing all node-based AAD applications:"
        for app in aad_app_list:
            buf = buf + app.show()
            buf = buf + "\n"
    else:
        buf = buf + "\nNo node-based AAD applications are configured"
    return buf



#!/usr/bin/env python

import os
import urllib
from os.path import expanduser
import requests
import traceback
import logging
from logging.handlers import RotatingFileHandler
from datetime import datetime, timedelta


# Specify files accessed by this script in guestshell
base_dir = expanduser('~') + '/cloud/authMgr/'
get_response_file = base_dir + "token_get_rsp"
debug_file = base_dir + "token_svr.log"
cert_file = "/etc/ssl/certs/ca-bundle.trust.crt"
token_file = base_dir + "token_file"


class AADAuth:

    def __doc__(self):
        """This class represents an application in Azure Active Directory"""

    def __init__(self, logger, cloud, tenant_id, app_id, app_key): #pragma: no cover
        self.logger = logger
        self.cloud = cloud
        self.app_id = app_id
        self.tenant_id = tenant_id
        self.token = ''
        self.token_expiry_time = datetime.utcnow() - timedelta(minutes=1)

        # Check if the app_key is URL encoded. If so, decode it
        if len(app_key) > 44 and app_key.count('%') > 0:
            self.app_key = urllib.unquote(app_key)
        else:
            self.app_key = app_key

        # Try to get a token
        token = self.get_token()
        if token == '':
            self.logger.error("AAD app failed verification: unable to obtain a token")

    def aad_get_token(self, cloud, tenant_id, app_id, app_key):
        if cloud == 'azure':
            url = "https://login.microsoftonline.com/%s/oauth2/token?api-version=1.1" % tenant_id
            payload = {'grant_type': 'client_credentials',
                       'client_id': app_id,
                       'resource': 'https://management.core.windows.net/',
                       'client_secret': app_key}

        elif cloud == 'azusgov':
            url = "https://login-us.microsoftonline.com/%s/oauth2/token?api-version=1.1" % tenant_id
            payload = {'grant_type': 'client_credentials',
                       'client_id': app_id,
                       'resource': 'https://management.core.usgovcloudapi.net/',
                       'client_secret': app_key}
        elif cloud == 'azchina':
            url = "https://login.chinacloudapi.cn/%s/oauth2/token?api-version=1.1" % tenant_id
            payload = {'grant_type': 'client_credentials',
                       'client_id': app_id,
                       'resource': 'https://management.core.chinacloudapi.cn/',
                       'client_secret': app_key}

        else:
            self.logger.error("Server: aad_get_token: invalid cloud name %s", cloud)
            return ''

        # Specify the HTTP POST request to obtain a token
        all_headers = {'Content-Type': 'application/x-www-form-urlencoded',
                       'Accept': 'accept:application/json',
                       'Authorization': 'OAuth 2.0'}

        # Send the HTTP POST request for the token
        try:
            response = requests.post(url, data=payload, verify=cert_file, headers=all_headers)
        except requests.exceptions.RequestException as e:
            self.logger.exception("Server: aad_get_token: request had error %s", str(e))
            return ''

        if 200 != response.status_code:
            self.logger.error("Server: aad_get_token: cloud: %s request failed rc=%d", cloud, response.status_code)
            with open(get_response_file, 'wb') as token_fh:
                for chunk in response.iter_content(chunk_size=64):
                    token_fh.write(chunk)
            return ''

        # Parse the HTTP GET response
        try:
            token = response.json()["access_token"]
            self.logger.info("Server: aad_get_token: cloud: {}, obtained token".format(cloud))

        except Exception as e:
            self.logger.exception("Server: aad_get_token:, cloud: %s caught exception %s", cloud, str(e))
            tb = traceback.format_exc()
            self.logger.exception("%s", tb)
            token = ''

        return token

    def get_token(self):
        if datetime.utcnow() < self.token_expiry_time:
            # Cached token is still valid. Just return it.
            return self.token
        else:
            # Need to refresh the token
            token = self.aad_get_token(self.cloud, self.tenant_id, self.app_id, self.app_key)
            if token != '':
                self.token = token
                self.token_expiry_time = datetime.utcnow() + timedelta(minutes=5)
                return self.token
            else:
                return ''

    def refresh_token(self):
        # Invalidate the current token
        self.token = ''
        self.token_expiry_time = datetime.utcnow() - timedelta(minutes=1)
        self.logger.info("Token successfully cleared")
        # Return a new token
        return self.get_token()

    def clear_token(self):
        self.token = ''
        self.token_expiry_time = datetime.utcnow()
        self.logger.info("Token successfully cleared")
        return 'OK'

    def show(self):
        buf = ''
        buf = buf + "\nApp ID:\t%s" % self.app_id
        buf = buf + "\nTenant ID:\t%s" % self.tenant_id
        buf = buf + "\nApp Key:\t%s" % self.app_key
        if self.token != '':
            buf = buf + "\nToken:\t%s" % self.token
            buf = buf + "\nExpires:\t%s" % self.token_expiry_time
            if self.token_expiry_time > datetime.utcnow():
                buf = buf + "\nToken is valid"
            else:
                buf = buf + "\nToken is expired"
        else:
            buf = buf + "\nNo token found"
        return buf



import os
import sys
import time
import logging
from logging.handlers import RotatingFileHandler
import socket
import subprocess
from os.path import expanduser

base_dir = expanduser('~') + '/cloud/authMgr/'
sock_file = base_dir + "sock_file"
log_file = base_dir + "token_api.log"

if not os.path.exists(base_dir):
    os.makedirs(base_dir)

logger = logging.getLogger("Token_api")
handler = RotatingFileHandler(log_file, maxBytes=1000000, backupCount=2)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s', datefmt='%m/%d/%Y %H:%M:%S')
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)


def open_socket(): #pragma: no cover
    global sock
    # Create a UDS socket
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

    # Connect the socket to the port where the server is listening
    try:
        sock.connect(sock_file)
    except socket.error, msg:
        logger.error("API: open_socket connect error %s" % msg)
        return (1, msg)
    return (0, sock)


def ping_server(sock):
    sock.settimeout(3)
    try:
        # Send data
        message = 'Ping'
        sock.sendall(message)

        # Receive the response
        data = sock.recv(8)
        if data == "Ack":
            return 0
        logger.error("API: ping_server: unexpected server response %s" % data)
        return 1

    except socket.timeout:
        logger.error("API: ping_server: socket timeout")
        return 2
    except socket.error as e:
        logger.error("API: ping_server: socket error %s" % str(e))
        return 3
    except Exception as e:
        logger.error("API: ping_server: other socket error %s" % str(e))
        return 4


def connect():
    global sock

    (rc, sock) = open_socket()
    if rc != 0:
        return rc

    rc = ping_server(sock)
    if rc != 0:
        logger.error("API: connect: ping failed")

    return rc


def send_message_to_server(message): # pragma: no cover
    global sock

    rc = connect()
    if rc != 0:
        logger.error("API: failed to connect to server")
        print "API: failed to connect to server"
        sys.exit(1)

    sock.settimeout(3)
    try:
        # Send data
        sock.sendall(message)

        # Receive the response
        data = sock.recv(80)

    except socket.timeout:
        logger.error("API: socket timeout")
        print "API: socket timeout"
    except socket.error:
        logger.error("API: socket error")
        print "API: socket error"
    except Exception as e:
        logger.error("API: other socket error %s" % str(e))
        print "API: other socket error %s" % str(e)

    disconnect()
    return data


def send_show_req_to_server(message): # pragma: no cover
    global sock

    rc = connect()
    if rc != 0:
        logger.error("API: failed to connect to server")
        print "API: failed to connect to server"
        sys.exit(1)

    sock.settimeout(3)
    buf = ''
    try:
        # Send data
        sock.sendall(message)

        # Receive the number of bytes in the token
        data = sock.recv(8)
        # Split the first response from the server
        two_str = data.rsplit('#')
        str_len = int(two_str[0])
        buf = two_str[1]

        # Now get the remaining bytes of the token
        total_bytes_received = len(two_str[1])
        while total_bytes_received < str_len:
            data = sock.recv(8)
            bytes_received = len(data)
            buf = buf + data
            total_bytes_received += bytes_received
        print "%s" % buf
        rc = 0

    except socket.timeout:
        logger.error("API: socket timeout")
        print "API: socket timeout"
        rc = 1
    except socket.error:
        logger.error("API: socket error")
        print "API: socket error"
        rc = 2
    except Exception as e:
        logger.error("API: other socket error %s" % str(e))
        print "API: other socket error %s" % str(e)
        rc = 3

    disconnect()
    return rc


def disconnect(): # pragma: no cover
    global sock
    sock.close()


def request_token_by_default_app(): # pragma: no cover
    global sock

    logger.info("Requesting token from default authentication application")
    token = ''
    token_len = 0
    total_bytes_received = 0
    try:
        # Send request message
        req_msg = "MSI_req"
        sock.sendall(req_msg)

        # Receive the number of bytes in the token
        data = sock.recv(8)
        # Split the first response from the server
        two_str = data.rsplit(' ')
        token_len = int(two_str[0])
        token = two_str[1]

        # Now get the remaining bytes of the token
        total_bytes_received = len(two_str[1])
        while total_bytes_received < token_len:
            data = sock.recv(8)
            bytes_received = len(data)
            token = token + data
            total_bytes_received += bytes_received

    except socket.timeout:
        logger.error("API: msi: socket timeout")
        logger.error("API: msi: expected token len %d" % token_len)
        logger.error("API: msi: actual token len %d" % total_bytes_received)
        token = ''
    except socket.error as e:
        logger.error("API: msi: socket error %s" % str(e))
        logger.error("API: msi: expected token len %d" % token_len)
        logger.error("API: msi: actual token len %d" % total_bytes_received)
        token = ''
    except Exception as e:
        logger.error("API: msi: other socket error %s" % str(e))
        logger.error("API: msi: expected token len %d" % token_len)
        logger.error("API: msi: actual token len %d" % total_bytes_received)
        token = ''

    finally:
        return token


def request_token_by_aad(cloud, tenantId, appId, appKey): #pragma: no cover
    global sock

    logger.info("Requesting token from specified AAD application")
    token = ''
    try:
        # Send request message
        req_msg = "AAD_req %s %s %s %s" % (cloud, tenantId, appId, appKey)
        sock.sendall(req_msg)

        # Receive the number of bytes in the token
        data = sock.recv(8)
        # Split the first response from the server
        two_str = data.rsplit(' ')
        token_len = int(two_str[0])
        token = two_str[1]
        token_len = token_len - len(two_str[1])

        # Now get the remaining bytes of the token
        total_bytes_received = 0
        while total_bytes_received < token_len:
            data = sock.recv(8)
            bytes_received = len(data)
            token = token + data
            total_bytes_received += bytes_received

    except socket.timeout:
        logger.error("API: aad: socket timeout")
        token = ''
    except socket.error as e:
        logger.error("API: aad: socket error %d" % str(e))
        token = ''
    except Exception as e:
        logger.error("API: aad: other socket error %d" % str(e))
        token = ''

    finally:
        return token


class Token_api():
    def __init__(self):
        if connect() != 0:
            subprocess.call(["sudo", "systemctl", "stop", "auth-token"])
            subprocess.call(["sudo", "systemctl", "start", "auth-token"])
            logger.error("Restarting the Token_svr")
            time.sleep(3)
            rc = connect()
            if rc != 0:
                logger.error("Unable to start the Token_svr")
                logger.error("Failed connection to Token Manager rc=%d" % rc)
                sys.exit()

    def set_default_aad_app(self, cloud, tenantId, appId, appKey):
        req_msg = "Set_app %s %s %s %s" % (cloud, tenantId, appId, appKey)
        return send_message_to_server(req_msg)

    def clear_default_aad_app(self):
        return send_message_to_server('Clear_app')

    def clear_aad_app_list(self):
        return send_message_to_server('Clear_list')

    def show_auth_apps(self):
        send_show_req_to_server('Show')
        return 'OK'

    def clear_token(self):
        return send_message_to_server('Clear_token')

    def refresh_token(self):
        return send_message_to_server('Refresh')

    def get_token_with_params(self, cloud, tenantId, appId, appKey):
        for attempt in range(3):
            rc = connect()
            if rc == 0:
                token = request_token_by_aad(cloud, tenantId, appId, appKey)

                if token == '':
                    logger.error("API: AAD: Failed to get token on attempt %d" % attempt)
                    disconnect()
                    if attempt < 3:
                        time.sleep(2)
                    else:
                        return token
                else:
                    disconnect()
                    break
            else:
                if attempt < 3:
                    time.sleep(2)
        else:
            return ''
        return token

    def get_token(self):
        for attempt in range(3):
            rc = connect()
            if rc == 0:
                token = request_token_by_default_app()

                if token == '':
                    logger.error("API: default: Failed to get token on attempt %d" % attempt)
                    disconnect()
                    if attempt < 3:
                        time.sleep(2)
                    else:
                        return token
                else:
                    disconnect()
                    break
            else:
                if attempt < 3:
                    time.sleep(2)
        else:
            return ''
        return token

    def disconnect(self): # pragma: no cover
        return disconnect()

    def is_server_up(self):
        return connect()





#!/usr/bin/env python

import os
import sys
from os.path import expanduser
import socket
#import urllib3.contrib.pyopenssl
import urllib3
import traceback
import logging
from logging.handlers import RotatingFileHandler
import auth_mgr

# Specify files accessed by this script in guestshell
base_dir = expanduser('~') + '/cloud/authMgr/'
debug_file = base_dir + "token_svr.log"
sock_file = base_dir + "sock_file"

if not os.path.exists(base_dir):
    os.makedirs(base_dir)

# Create and configure a logger for the token server
logger = logging.getLogger("Token_svr")
handler = RotatingFileHandler(debug_file, maxBytes=1000000, backupCount=2)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s', datefmt='%m/%d/%Y %H:%M:%S')
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)


def start_token_svr(): #pragma: no cover
    # Find out the process ID
    pid = os.getpid()

    # Open the debug log file
    logger.info("Token server started, pid=%d", pid)
    # Make sure the socket does not already exist
    try:
        os.unlink(sock_file)
    except OSError:
        if os.path.exists(sock_file):
            raise

    # Create a UDS socket
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

    # Bind the socket to the port
    sock.bind(sock_file)

    # Use OpenSSL to check certificates for https
    urllib3.contrib.pyopenssl.inject_into_urllib3()

    # Create the MSI based authentication application
    try: 
        auth_mgr.set_default_msi_app(logger)
    except Exception as e:
        logger.error("Server: failed to enable authentication using MSI: %s" % str(e))

    # Get default AAD application if it exists in the app_file
    auth_mgr.read_table_from_file(logger)

    # Listen for incoming connections
    sock.listen(1)

    while True:
        # Wait for a connection
        connection, client_address = sock.accept()
        try:
            while True:
                # Read a command from the client
                line = connection.recv(255)
                if line == '':
                    break

                command = line.rsplit(' ')
                if command[0] == "MSI_req":
                    # Ask the authentication manager for a token
                    token = auth_mgr.get_token(logger)

                    # Send the number of bytes in the token
                    token_len = len(token)
                    token_len_as_str = str(token_len) + ' '
                    connection.sendall(token_len_as_str)

                    # Send the token
                    if token != '':
                        connection.sendall(token)
                    else:
                        logger.error("Server: no token returned")

                elif command[0] == "AAD_req":
                    token = ''
                    # Validate the AAD app parameters and get the app
                    aad_app = auth_mgr.verify_aad_app(logger, command[1], command[2], command[3], command[4])
                    if aad_app is not None:
                        # Ask the authentication manager for a token
                        token = auth_mgr.get_token(logger, aad_app)

                    # Send the number of bytes in the token
                    token_len = len(token)
                    token_len_as_str = str(token_len) + ' '
                    connection.sendall(token_len_as_str)

                    # Send the token
                    if token != '':
                        connection.sendall(token)
                    else:
                        logger.error("Server: no token returned by AAD")

                elif command[0] == "Ping":
                    connection.sendall("Ack")
                elif command[0] == 'MSI_app':
                    rc = auth_mgr.set_default_msi_app(logger)
                    connection.sendall(rc)
                elif command[0] == 'Set_app':
                    rc = auth_mgr.set_default_aad_app(logger, command[1], command[2], command[3], command[4])
                    connection.sendall(rc)
                elif command[0] == 'Clear_app':
                    rc = auth_mgr.clear_default_aad_app()
                    connection.sendall(rc)
                elif command[0] == 'Clear_list':
                    rc = auth_mgr.clear_aad_app_list()
                    connection.sendall(rc)
                elif command[0] == 'Show':
                    buf = auth_mgr.show()
                    buf_len_as_str = str(len(buf) - 1) + '#'
                    connection.sendall(buf_len_as_str)
                    connection.sendall(buf)
                elif command[0] == 'Clear_token':
                    rc = auth_mgr.clear_token(logger)
                    connection.sendall(rc)
                elif command[0] == 'Refresh':
                    token = auth_mgr.refresh_token(logger)
                    if token == '':
                        rc = 'ERR: Token not refreshed'
                    else:
                        rc = 'OK'
                    connection.sendall(rc)
                else:
                    logger.error("Token server unrecognized command %s", command)
                    err_msg = "Error: invalid command %s" % command
                    connection.sendall(err_msg)

            # Clean up the connection
            connection.close()

        except Exception as e:
            logger.error("Token server caught exception %s", str(e))
            tb = traceback.format_exc()
            logger.error("%s", tb)
            #connection.sendall(e.message)
            connection.close()


if __name__ == "__main__":
    if len(sys.argv) < 2:
        logger.error("Incorrect usage of token server script")
        logger.info("Usage: {} start".format(__file__))
        sys.exit(1)
    if sys.argv[1] == 'start':
        start_token_svr()



#!/usr/bin/env python

import sys
from csr_cloud.token_api import Token_api as token_api
import argparse


def main(argv):

    parser = argparse.ArgumentParser(description="AAD Application")
    parser._action_groups.pop()
    required = parser.add_argument_group('required arguments')
    optional = parser.add_argument_group('optional arguments')

    required.add_argument('-p', help='<cloud_provider>   {azure | azusgov | azchina}',
                          choices=['azure', 'azusgov', 'azchina'], default=None, required=True)
    required.add_argument('-a', help='to add the applicationId', default=None, required=True)
    required.add_argument('-d', help='to add the tenantId', default=None, required=True)
    required.add_argument('-k', help='to add the applicationKey', default=None, required=True)

    args = parser.parse_args()

    # Establish a connection to the token server
    conn = token_api()

    # Verify server is up
    rc = conn.is_server_up()
    if rc != 0:
        print "failed connection to token manager rc=%d" % rc
        sys.exit()

    # Request server to clear the default AAD application
    result = conn.set_default_aad_app(args.p, args.d, args.a, args.k)
    if result != 'OK':
        print "Failed to set default AAD application: %s" % result
        sys.exit()

    conn.disconnect()


if __name__ == '__main__':
    sys.exit(main(sys.argv))




from file_utils import StorageFileUtils
from meta_utils import MetaDataUtils
from general_utils import GeneralUtils

class AzureUtils(StorageFileUtils, MetaDataUtils, GeneralUtils):
    def __init__(self, account_name, account_key, cloudname, feature):
        StorageFileUtils.__init__(self, account_name, account_key, cloudname, feature)
        MetaDataUtils.__init__(self, feature)
        GeneralUtils.__init__(self)




'''
Cisco Copyright 2018
Author: Vishlesh Patel <vishlpa4@cisco.com>

FILENAME: METRIC_UTILS.PY
'''

import logging
import os
from applicationinsights import TelemetryClient
from azure.common.credentials import ServicePrincipalCredentials
from azure.mgmt.applicationinsights import ApplicationInsightsManagementClient
from azure.mgmt.monitor import MonitorManagementClient
import urlparse
import requests
import configparser

AI_URLBASE = "https://api.applicationinsights.io/{version}/apps/{appId}/{operation}/"
API_VERSION = 'v1'

class MetricUtils:

    def __init__(self, feature=None):
        self.feature = feature if feature is not None else __name__
        self.logger = logging.getLogger(self.feature)
        self.apiKey = os.getenv("API_KEY", '')
        self.appId = os.getenv("APP_ID", '')
        self.instrumentation_key = os.getenv("INSTRUMENTATION_KEY","")
        self.tc = TelemetryClient(self.instrumentation_key)
        self.region = os.getenv('DEFAULT_REGION', "")
        self.app_id = os.getenv("AZURE_CLIENT_ID", "")
        self.app_key = os.getenv("AZURE_CLIENT_SECRET", "")
        self.subscription_id = os.getenv("SUBSCRIPTION_ID", "")
        self.customer_resource_group = os.getenv("CUSTOMER_RESOURCE_GROUP", "")
        self.tenant = os.getenv("TENANT_ID", "")
        if self.app_id != "" and self.app_key != "" and self.tenant != "" and self.subscription_id != "":
            self.create_clients()

    def create_clients(self):
        self.credentials = ServicePrincipalCredentials(
            client_id=self.app_id,
            secret=self.app_key,
            tenant=self.tenant
        )
        self.appInsights_client = ApplicationInsightsManagementClient(credentials=self.credentials,subscription_id=self.subscription_id)
        self.monitor_management_client = MonitorManagementClient(self.credentials, self.subscription_id)

    def create_app_insights_resource(self, resource_name):
        insights_properties = {}
        insights_properties['location'] = self.region
        insights_properties['kind'] = 'other'
        insights_properties['application_type'] = 'other'
        insights_properties['flow_type'] = 'Bluefield'
        insights_properties['request_source'] = 'rest'
        try:
            response = self.appInsights_client.components.create_or_update(self.customer_resource_group,resource_name,insights_properties)
            self.logger.info("App Insight resource created successfully: %s" % response)
            return response.instrumentation_key, response.app_id
        except Exception as e:
            self.logger.exception("Exception: %s" % e)
            return None

    def create_api_key(self,resource_name):
        try:
            api_params = {}
            api_params['name'] = resource_name + '-api-key'
            url = '/subscriptions/' + self.subscription_id + '/resourcegroups/' + self.customer_resource_group + '/providers/Microsoft.insights/components/' + resource_name + '/api'
            api_params['linked_read_properties'] = [url]
            api_key = self.appInsights_client.api_keys.create(self.customer_resource_group, resource_name, api_params)
            self.logger.info("API key created successfully: %s" % api_key)
            return api_key
        except Exception as e:
            self.logger.exception("Exception: %s" % e)
            return None

    def load_env(self):
        try:
            config = configparser.ConfigParser()
            config.optionxform = str
            home = os.path.expanduser("~")
            filename = home + '/.azure/credentials'
            config.read(filename)
            if os.path.exists(filename):
                for key in config['default']:
                    value = config.get('default', key)
                    if value.startswith('"') and value.endswith('"'):
                        value = value[1:-1]
                    if value.startswith("'") and value.endswith("'"):
                        value = value[1:-1]
                    os.environ[key] = str(value)
        except Exception as e:
            self.logger.exception(e)

    def put_metric(self, metricName, value, dimensions=None):
        try:
            self.tc.track_metric(metricName, value, properties=dimensions)
        except Exception as e:
            self.logger.exception(e)

    def put_event(self, eventName, value=None, dimenstions=None):
        measurements = None
        if value is not None:
            measurements = {
                'value' : value
            }
        try:
            self.tc.track_event(eventName, measurements, dimenstions)
        except Exception as e:
            self.logger.exception(e)

    def get_az_time(self, time_period):
        '''

        :param time_period: time period in seconds
        :return:
        '''
        t = None
        time_period = int(time_period)
        if time_period < 60:
            t = "PT" + str(time_period) + "S"
        elif time_period < 3600:
            time_period = int(time_period / 60)
            t = "PT" + str(time_period) + "M"
        elif time_period < 21600:
            time_period = int(time_period / 3600)
            t = "PT" + str(time_period) + "H"
        else:
            time_period = int(time_period / 21600)
            t = "P" + str(time_period) + "D"
        return t

    # def get_azure_metric(self, vm_name, metric, minutes):
    def get_azure_metric(self,
                   metricName,
                   period=60,
                   metric_period=60,
                   aggregation_type='avg',
                   resource_id=None):

        try:
            timespan = self.get_az_time(period)
            interval = self.get_az_time(metric_period)

            metrics_data = self.monitor_management_client.metrics.list(
                resource_id,
                timespan=timespan,
                interval=interval,
                aggregation='Total',
                metricnames=metricName
            )
        except Exception as e:
            self.logger.exception("Exception while retrieving metric %s: %s" % (metricName, e))
            return
        # build up list to return
        mylist = []

        for item in metrics_data.value:
            for timeserie in item.timeseries:
                for data in timeserie.data:
                    if aggregation_type.lower() == 'avg':
                        data.total = data.total / metric_period
                    metric_tuple = (metricName, data.time_stamp, data.total)
                    mylist.append(metric_tuple)

        return mylist

    def get_custom_metric(self,
                   metricName,
                   metricType='customMetrics',
                   period=60,
                   metric_period=60,
                   aggregation_type='avg',
                   dimensions=None):
        '''
        :param name: name of the metric to retrieve
        :param metricType - Type of metric
        :param timespan - timespan period - PT1D (Past 1 day), or PT1H (Past 1 Hour) or PT1M (Past 1 Minute)
        :param interval - interval period - PT1D (Past 1 day), or PT1H (Past 1 Hour) or PT1M (Past 1 Minute)
        :param dimesions: dimensions dict if you want to retrieve metric for only that dimensions
            for e.g.,
            if you want to retrieve metrics only for CSR whose instanceID == 1
            dimensions dict --> { 'instanceID' : '1' }
        :return: returns metrics collected in dict format
        '''
        try:
            metricPath = metricType + '/' + metricName
            aggregation_type = aggregation_type.lower()
            dimensionsKey = None
            dimensionsVal = None
            if len(dimensions) > 0:
                keys = dimensions.keys()
                dimensionsKey = keys[0]
                dimensionsVal = dimensions[keys[0]]
            new_url = AI_URLBASE.format(
                version=API_VERSION, appId=self.appId, operation='metrics')
            url = urlparse.urljoin(new_url, metricPath)
            timespan = self.get_az_time(period)
            interval = self.get_az_time(metric_period)

            query = 'timespan={}&interval={}&aggregation={}'.format(
                timespan, interval, aggregation_type)
            if dimensionsKey is not None and dimensionsVal is not None:
                query += "&filter=customDimensions/{}%20eq%20'{}'".format(dimensionsKey, dimensionsVal)
            url += '?' + query
            self.logger.debug("url being used to retrieve metrics {}".format(url))
            metrics = requests.get(url, headers={"X-Api-Key": self.apiKey})
            if metrics.status_code == 200:
                self.logger.debug("Get Metric request status code - 200")
            else:
                self.logger.warning("bad return status code {} when trying to retrieve {}, dimension {}, \
                aggregation type {}".format(metrics.status_code, metricPath, dimensionsVal, aggregation_type))

            mylist = []
            metrics = metrics.json()
            for segment in metrics['value']['segments']:
                metricPath = 'customMetrics/{}'.format(metricName)
                response_value = segment[metricPath][aggregation_type]
                timestamp = segment["start"]
                mylist.append((metricName, timestamp, response_value))

            return mylist

        except Exception as e:
            self.logger.exception(e)
            return None

    def get_event_metrics(self,
                          name,
                          eventType='customEvents',
                          period='PT24H',
                          query=None):
        '''

        :param name: Name of event
        :param eventType: Event type. e.g. customEvents, traces, requests
        :param period: Timespan
        :param query: Search Query
        :return:
        '''
        try:
            new_url = AI_URLBASE.format(
                version=API_VERSION, appId=self.appId, operation='metrics')
            url = urlparse.urljoin(new_url, eventType)
            q = '?timespan={}'.format(period)
            q += "&$filter=customEvent/name eq '{}'".format(name)
            if query is not None:
                q += '&' + query

            url += '?' + q
            self.logger.debug("url being used to retrieve events {}".format(url))
            self.logger.debug(self.apiKey)
            events = requests.get(url, headers={"X-Api-Key": self.apiKey}).text
            if events.status_code == 200:
                self.logger.debug("Get event request status code - 200")
            else:
                self.logger.warning("bad return status code {}".format(events.status_code))
            return events.json()
        except Exception as e:
            self.logger.exception(e)

    def flush_metric_queue(self):
        try:
            self.tc.flush()
        except Exception as e:
            self.logger.exception(e)



#!/usr/bin/env python

'''
Cisco Copyright 2018
Author: Abhinav Sanakkayala <asanakka@cisco.com>




'''
from azure.storage.blob import BlockBlobService
from azure.storage.blob import ContentSettings
import logging

log = logging.getLogger('csr_tvnet')

class BlobUtils():
    def __init__(self, account_name, account_key):

        self.block_blob_service = BlockBlobService(
            account_name=account_name, account_key=account_key)

    def create_container_if_doesnt_exist(self, containername):
        if self.does_container_exist(containername) is False:
            self.block_blob_service.create_container(containername)

    def download_file_from_container(self, containername, filename, directory="/bootflash/"):
        try:
            self.block_blob_service.get_blob_to_path(
                containername, filename, directory + filename)
        except Exception as e:
            log.exception("Config File Download Failed.  Error: %s" % e)
            return False
        print("\nDownload Complete")
        return True

    def upload_file_to_container(self, containername, filename, directory="/bootflash/"):
        self.create_container_if_doesnt_exist(containername)
        try:
            self.block_blob_service.create_blob_from_path(
                containername,
                filename,
                directory + filename,
                content_settings=ContentSettings(
                    content_encoding='UTF-8', content_language='en')
            )
        except Exception as e:
            print("Uploading %s Failed.  Error: %s" % (filename, e))
            return False

        print("Upload Complete to container %s" % (containername))
        return True

    def does_container_exist(self, containername):
        containers = self.block_blob_service.list_containers()

        for c in containers:
            if containername is c.name:
                return True
        return False



import os
import sys
import requests
import urllib
from os.path import expanduser
from token_api import Token_api as token_api
from meta_utils import MetaDataUtils as Metadata
import logging
import ipaddress
from logging.handlers import RotatingFileHandler
import datetime
import json
import shutil

log = logging.getLogger('HA')
log.setLevel(logging.INFO)

home = expanduser("~")
event_dir = "%s/cloud/HA/events" % home


class csr_ha():
    def __init__(self):                                                                 #pragma nocover
        self.cert_file = "/etc/ssl/certs/ca-bundle.trust.crt"
        sys.path.append(home + '/.local/lib/python2.7/site-packages/csr_ha/client_api')
        sys.path.append(home + '.local/lib/python2.7/site-packages/csr_ha/server')

        # Setting event_logger for route events for fail-safe approach
        # Ideally csr_ha is responsible for setting it up using set_event_logger
        self.event_logger = log
        self.event_log_file = ""

        if os.path.exists("/home/guestshell/cloud/HA/csr_ha.log"):
            if os.path.getsize("/home/guestshell/cloud/HA/csr_ha.log") > 15000000:
                shutil.copy('/home/guestshell/cloud/HA/csr_ha.log', '/bootflash/csr_ha.log')
                os.remove('/home/guestshell/cloud/HA/csr_ha.log')


    def create_event_logger(self, node, event_type, directory_name):
        '''
            This function will create the event logger
        '''

        # Create the logger for events
        logger = logging.getLogger("HA.events")

        # Name the log file
        log_file = "node_" + str(node['index']) + "_" + str(
            datetime.datetime.now().strftime("%Y-%m-%d_%H:%M:%S")) + "_" + event_type

        # save the file path so that we can remove the file in-case of unnecessary reverts
        self.event_log_file = os.path.join(directory_name, log_file)

        # Roll over if the number of event files exceeds 513 count
        if len(os.listdir(directory_name)) > 512:
            filetoremove = os.popen("ls -t " + directory_name + "|  tail -1").read().strip()
            log.info('Event files threshold exceeded. Removing event file: "{}"'.format(os.path.join(directory_name, filetoremove).strip()))
            command = "rm {}".format(os.path.join(directory_name, filetoremove))
            os.system(command)

        # Create the file handler
        handler = logging.FileHandler(filename=os.path.join(directory_name, log_file), mode='a')

        log.info("Created event_log file: {} for node {} {} event".format(self.event_log_file, node['index'], event_type))
        # Add Handler
        if not len(logger.handlers):
            logger.addHandler(handler)

        return logger

    def set_event_logger(self, node, event_type, directory_name=event_dir):
        '''
            This function will set the event_logger for ha
        '''

        self.event_logger = self.create_event_logger(node, event_type, directory_name)


    def obtain_token(self, node, event_type):
        # connect to the token manager

        rc = token_api()
        token = ""
        server_status = rc.is_server_up()

        if server_status != 0:
            self.event_logger.info("failed connection to token manager rc=%d" % rc)
            sys.exit()

        if 'appId' in node:
            # appid has been specified. use aad to get a token
            token = rc.get_token_with_params(node['cloud'],
                                             node['tenantId'],
                                             node['appId'],
                                             node['appKey'])
            self.event_logger.info("requesting token using Azure Active Directory")
            if event_type == 'verify':
                self.event_logger.info("token=%s" % token)

            if token is "":
                self.event_logger.info("failed to obtain token using AAD")

        else:
            # no appid specified.  use msi to get token.
            token = rc.get_token()
            self.event_logger.info("Requesting token from default authentication application")
            if event_type == 'verify':
                self.event_logger.info("token=%s" % token)

            if token is "":
                self.event_logger.info("failed to obtain token using default application")

        rc.disconnect()
        if token == '':
            self.event_logger.info("failed to obtain token")
        return token


    def get_route_table(self, node, event_type=None):

        self.event_logger.info("Requesting token for fetching the routes from route table")
        token = self.obtain_token(node, event_type)
        total_nodes = ""

        if token == "":
            self.event_logger.error("Failed to obtain token")
            return None
        else:
            self.event_logger.info("Obtained token successfully")

        # Specify the HTTP GET request to read the route table
        apiversion = "2017-10-01"
        auth_header = "Bearer " + token
        all_headers = {'Content-Type': 'application/x-www-form-urlencoded',
                       'Accept': 'application/json',
                       'Authorization': auth_header}

        if node['cloud'] == 'azure':
            url = "https://management.azure.com/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], node['routeTableName'], apiversion)
        elif node['cloud'] == 'azusgov':
            url = "https://management.usgovcloudapi.net/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], node['routeTableName'], apiversion)
        elif node['cloud'] == 'azchina':
            url = "https://management.chinacloudapi.cn/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], node['routeTableName'], apiversion)
        else:
            self.event_logger.error("Unknown cloud name %s" % node['cloud'])
            return None

        # Send the HTTP GET request for the route table
        try:
            response = requests.get(url, verify=self.cert_file, headers=all_headers)
        except requests.exceptions.RequestException as e:
            self.event_logger.error("get_route_table: request had error %s" % e)
            return None

        if event_type == 'verify':
            # Write the HTTP GET response to the event file for debugging purposes
            self.event_logger.info("Fetching the route table")
            self.event_logger.info("Route table get response:\n {r}".format(r=response.text))

        if 200 == response.status_code:
            self.event_logger.info("Read route table successfully")
        else:
            self.event_logger.error("Route GET request failed with code %d" % response.status_code)
            self.event_logger.info("Route table get response:\n {r}".format(r=response.text))
            return None

        # Extract the routes section from the table
        route_table = response.json()
        routes = route_table['properties']['routes']

        if routes == '':
            self.event_logger.error("No routes found in table")
            self.event_logger.error("Get route response:\n{r}".format(r=route_table))
            return None

        total_routes = ""
        for i, route in enumerate(route_table["properties"]["routes"]):
            total_routes +="\n{:20s}{}".format(str(route['properties']['addressPrefix']), str(route['properties']['nextHopIpAddress']))
        if event_type == 'verify':
            self.event_logger.info("\nRoutes on route table {}:  {}\n".format(node['routeTableName'], total_routes))
        return route_table


    def set_one_route(self, node, event_type, route_table):
        self.event_logger.info("Requesting token for setting the route")
        token = self.obtain_token(node, event_type)

        if token == "":
            self.event_logger.error("Failed to obtain token")
            return
        else:
            self.event_logger.info("Obtained token successfully")

        if event_type == 'verify':
            self.event_logger.info("Evaluating single route in route table for event type %s" % event_type)

        if route_table == '':
            self.event_logger.info("No route table entries found")
            if event_type == 'verify':
                self.event_logger.info("It is likely permission to access the route table was not granted.")
            return

        send_request = False
        found_route = False

        # Walk through all the routes in the current route table
        for i, route in enumerate(route_table["properties"]["routes"]):
            if route['properties']['nextHopType'] == 'VirtualAppliance':
                # Updating a single route in the table.  Need to find the right one.
                if node['route'] == route['properties']['addressPrefix']:
                    # This is the one
                    found_route = True
                    if event_type == 'verify':
                        # Don't change the route
                        newNextHop = route['properties']['nextHopIpAddress']
                        send_request = True
                    elif event_type == 'peerFail':
                        newNextHop = node['nextHop']
                        send_request = True
                    elif event_type == 'revert':
                        newNextHop = node['nextHop']
                        if 'mode' in node:
                            if node['mode'] == 'primary':
                                if route['properties']['nextHopIpAddress'] != node['nextHop']:
                                    send_request = True
                                    self.event_logger.info("Revert: updating %s with nextHop %s as mode configured for primary" %
                                        (route['properties']['addressPrefix'], node['nextHop']))
                    else:
                        self.event_logger.error("Invalid event type %s in set_route_table" % event_type)
                        return
                    break

        if send_request == False:
            if event_type == 'verify':
                self.event_logger.info("Verify event: No need to update single route in route table")
                return
            if event_type == 'revert':
                # No action taken by this revert event. Delete the event log file
                log.info("Removing the event_log_file: {}".format(self.event_log_file))
                os_command = "rm \"%s\"" % self.event_log_file
                os.system(os_command)
                return

        # Did we find the route?
        if found_route == False:
            self.event_logger.error("Did not find route %s event type %s" % (node['route'], event_type))
            return

            # Specify the HTTP PUT request to write the route table
        # Set the URL
        apiversion = "2017-10-01"
        if node['cloud'] == 'azure':
            url = "https://management.azure.com/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s/routes/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], node['routeTableName'], route['name'], apiversion)
        elif node['cloud'] == 'azusgov':
            url = "https://management.usgovcloudapi.net/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s/routes/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], node['routeTableName'], route['name'], apiversion)
        elif node['cloud'] == 'azchina':
            url = "https://management.chinacloudapi.cn/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s/routes/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], node['routeTableName'], route['name'], apiversion)

        else:
            self.event_logger.error("Unknown cloud name %s" % node['cloud'])
            return
            # Set the headers
        auth_header = "Bearer " + token
        all_headers = {'Content-Type': 'application/json',
                       'Accept': 'application/json',
                       'Authorization': auth_header}

        # Build the payload
        payload = "{\"properties\":{\"addressPrefix\":\"%s\", \"nextHopType\":\"%s\", \"nextHopIpAddress\":\"%s\"}}" % (
            route['properties']['addressPrefix'], route['properties']['nextHopType'], newNextHop)

        if event_type == 'verify':
            self.event_logger.info("Updating single route in route table")
            self.event_logger.info("URL=%s" % url)
            for key in all_headers:
                self.event_logger.info("%s:%s" % (key, all_headers[key]))
            self.event_logger.info("Payload=%s" % payload)

        # Send the PUT request
        try:
            response = requests.put(url, data=payload, verify=self.cert_file, headers=all_headers)
        except requests.exceptions.RequestException as e:
            self.event_logger.error("set_one_route: request had error %s" % e)
            return

        write_rsp_to_file = False
        if 200 == response.status_code:
            self.event_logger.info("HTTP PUT of route table was successful")
            self.event_logger.info("Updated route %s to %s" % (route['name'], newNextHop))
            # Write the HTTP SET response to a file for debugging purposes
            if event_type == 'verify':
                write_rsp_to_file = True

            # Extract the routes section from the table
            put_response = response.json()
            provisionState = put_response['properties']['provisioningState']

            if event_type == 'verify':
                self.event_logger.info("Set route provision state is %s" % provisionState)

            if provisionState == 'Failed':
                self.event_logger.error("Set route provisioning state failed")
                write_rsp_to_file = True
        else:
            self.event_logger.error("Set route failed for route %s code=%d" %
                          (route['name'], response.status_code))
            write_rsp_to_file = True

        if write_rsp_to_file == True:
            self.event_logger.info("Set route response")
            self.event_logger.info(response.text)

        return response.status_code


    def set_all_routes(self, node, event_type, route_table):
        self.event_logger.info("Requesting token for setting all routes in the route table")
        token = self.obtain_token(node, event_type)

        if token == "":
            self.event_logger.error("Failed to obtain token")
            return
        else:
            self.event_logger.info("Obtained token successfully")
        if event_type == 'verify':
            self.event_logger.info("Evaluating all routes in route table for event type %s" % event_type)

        send_request = False

        if route_table == '':
            self.event_logger.info("No route table entries found")
            if event_type == 'verify':
                self.event_logger.info("It is likely permission to access the route table was not granted.")
            return

        # The etag can contain quotation marks which must be escaped
        current_etag = route_table['etag']
        new_etag = ''
        tag_len = len(current_etag)
        for i in range(tag_len):
            if current_etag[i] == '\"':
                new_etag = new_etag + '\\'
            new_etag = new_etag + current_etag[i]

        payload = "{\"location\":\"%s\", \"etag\":\"%s\", \"properties\":{\"routes\":[" % (
            route_table['location'], new_etag)

        # Walk through all the routes in the current route table
        routes = route_table['properties']['routes']
        for i, route in enumerate(routes):
            if route['properties']['nextHopType'] == 'VirtualAppliance':
                if event_type == 'verify':
                    # Don't change the route
                    newNextHop = route['properties']['nextHopIpAddress']
                    send_request = True
                elif event_type == 'peerFail':
                    newNextHop = node['nextHop']
                    send_request = True
                elif event_type == 'revert':
                    newNextHop = node['nextHop']
                    if 'mode' in node:
                        if node['mode'] == 'primary':
                            if route['properties']['nextHopIpAddress'] != node['nextHop']:
                                send_request = True
                                self.event_logger.info("Revert: updating %s with nextHop %s as mode configured for primary" %
                                    (route['properties']['addressPrefix'], node['nextHop']))

                else:
                    self.event_logger.error(" Invalid event type %s in set_route_table" % event_type)
                    return

                payload = payload + "{\"name\":\"%s\", \"id\":\"%s\", \"properties\":{\"addressPrefix\":\"%s\", \"nextHopType\":\"%s\", \"nextHopIpAddress\":\"%s\"}}," % (
                    route['name'], route['id'], route['properties']['addressPrefix'], route['properties']['nextHopType'],
                    newNextHop)

            else:
                # These route types have no explicit next hop IP address
                payload = payload + "{\"name\":\"%s\", \"id\":\"%s\", \"properties\":{\"addressPrefix\":\"%s\", \"nextHopType\":\"%s\"}}," % (
                    route['name'], route['id'], route['properties']['addressPrefix'], route['properties']['nextHopType'])

        payload = payload + ']}}'

        if send_request == False:
            if event_type == 'verify':
                self.event_logger.info("No need to update any routes in route table")
            if event_type == 'revert':
                # No action taken by this revert event. Delete the event log file
                os_command = "rm \"%s\"" % self.event_log_file
                os.system(os_command)
            return

            # Specify the HTTP PUT request to write the route table
        # Set the URL
        apiversion = "2017-10-01"
        if node['cloud'] == 'azure':
            url = "https://management.azure.com/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], route_table['name'], apiversion)
        elif node['cloud'] == 'azusgov':
            url = "https://management.usgovcloudapi.net/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], route_table['name'], apiversion)
        elif node['cloud'] == 'azchina':
            url = "https://management.chinacloudapi.cn/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/routeTables/%s?api-version=%s" % (
                node['subscriptionId'], node['resourceGroup'], route_table['name'], apiversion)
        else:
            self.event_logger.error("Unknown cloud name %s" % node['cloud'])
            return

            # Set the headers
        auth_header = "Bearer " + token
        all_headers = {'Content-Type': 'application/json',
                       'Accept': 'application/json',
                       'Authorization': auth_header}
        if event_type == 'verify':
            self.event_logger.info("URL=%s" % url)
            self.event_logger.info("Headers=%s" % all_headers)
            self.event_logger.info("Payload=%s" % payload)

        # Send the PUT request
        self.event_logger.info("Updating all routes in route table")
        try:
            response = requests.put(url, headers=all_headers, data=payload, verify=self.cert_file)
        except requests.exceptions.RequestException as e:
            self.event_logger.error("set_all_routes: request had error %s" % e)
            return

        write_rsp_to_file = False
        if 200 == response.status_code:
            # Write the HTTP SET response to a file for debugging purposes
            self.event_logger.info("HTTP PUT of route table was successful")
            self.event_logger.info("Updated all routes in table %s to %s" %
                          (node['routeTableName'], newNextHop))
            if event_type == 'verify':
                write_rsp_to_file = True

            # Extract the routes section from the table
            put_response = response.json()
            provisionState = put_response['properties']['provisioningState']

            if event_type == 'verify':
                self.event_logger.info("Set route provision state is %s" % provisionState)

            if provisionState == 'Failed':
                self.event_logger.error("Set route provisioning state failed")
                write_rsp_to_file = True
        else:
            self.event_logger.error("Failed to set all routes in table %s code=%d" %
                          (node['routeTableName'], response.status_code))
            write_rsp_to_file = True

        write_rsp_content = ""
        if write_rsp_to_file == True:
            self.event_logger.info("Set route response:\n%s" % response.txt)

        return response.status_code


    def set_route_table(self, node, event_type, route_table):
        # Are we changing all route entries, or just a specific one?
        if not route_table:
            self.event_logger.error("Route Table not found")
            rsp_code = 404

        if 'route' in node:
            # Updating a specific route in the table
            rsp_code = self.set_one_route(node, event_type, route_table)
            return rsp_code
        else:
            # Updating all routes in the table
            rsp_code = self.set_all_routes(node, event_type, route_table)
            return rsp_code


    def verify_node(self, node, event_type):
        param_str = " "
        self.event_logger.info("Redundancy node configuration: ")
        for key in node:
            param_str = param_str + "{:15s}{} \n".format(key, node[key])
        self.event_logger.info(param_str)

        if event_type == 'verify':
            node_verified = True
            if (not (node.has_key('cloud'))):
                self.event_logger.info("Missing required parameter -p cloud")
                node_verified = False
            if (not (node.has_key('subscriptionId'))):
                self.event_logger.info("Missing required parameter -s subscriptionId")
                node_verified = False
            if (not (node.has_key('resourceGroup'))):
                self.event_logger.info("Missing required parameter -g resourceGroup")
                node_verified = False
            if (not (node.has_key('routeTableName'))):
                self.event_logger.info("Missing required parameter -t routeTableName")
                node_verified = False
            if (not (node.has_key('nextHop'))):
                self.event_logger.info("Missing required parameter -n nextHopIpAddress")
                node_verified = False
            if node.has_key('appId'):
                if (not (node.has_key('tenantId'))):
                    self.event_logger.info("Missing required parameter -d tenantId")
                    node_verified = False
                if (not (node.has_key('appKey'))):
                    self.event_logger.info("Missing required parameter -k applicationKey")
                    node_verified = False

            if node_verified:
                self.event_logger.info("All required parameters have been provided")
                return 'OK'
            else:
                return 'ERR1'
        else:
            if ((node.has_key('index')) and
                    (node.has_key('cloud')) and
                    (node.has_key('routeTableName')) and
                    (node.has_key('subscriptionId')) and
                    (node.has_key('resourceGroup')) and
                    (node.has_key('nextHop'))):

                if node.has_key('appId'):
                    if (not (node.has_key('tenantId'))):
                        return "ERR1"
                    elif (not (node.has_key('appKey'))):
                        return "ERR1"
                return 'OK'
            else:
                self.event_logger.info("verify_node: missing required parameter")
                return "ERR1"

    def check_cloud_command(self, cmd):
        i = 1
        keyword = ""
        if cmd == '-s':
            keyword = "subscriptionId"
        elif cmd == '-r':
            keyword = 'route'
        elif cmd == '-p':
            keyword = "cloud"
        elif cmd == '-n':
            keyword = "nextHop"
        elif cmd == '-g':
            keyword = "resourceGroup"
        elif cmd == '-t':
            keyword = "routeTableName"
        elif cmd == '-a':
            keyword = "appId"
        elif cmd == '-d':
            keyword = "tenantId"
        elif cmd == '-k':
            keyword = "appKey"
        else:
            log.error("Invalid command format %s" % cmd)
            print "Invalid command format %s" % cmd
            return "Error"

        return keyword


    def create_node(self, params):
        # Make any cloud specific changes to the parameters
        node = params

        if params.has_key('appKey'):
            # Check if the key is URL encoded. If so, decode it.
            app_key = params['appKey']
            if len(app_key) > 44 and app_key.count('%') > 0:
                node['appKey'] = urllib.unquote(app_key)

        return node


    def check_clear_param(self, index,old_params):

        keyword = " "
        if old_params[index] == '-r':
            keyword = "route"
        elif old_params[index] == '-s':
            keyword = "subscriptionId"
        elif old_params[index] == '-g':
            keyword = "resourceGroup"
        elif old_params[index] == '-t':
            keyword = "routeTableName"
        elif old_params[index] == '-a':
            keyword = "appId"
        elif old_params[index] == '-n':
            keyword = "nextHop"
        elif old_params[index] == '-d':
            keyword = "tenantId"
        elif old_params[index] == '-k':
            keyword = "appKey"
        else:
            log.info("Invalid parameter format %s" % old_params[index])

        return keyword

    def clear_param_parser(self, optional, required):

        optional.add_argument('-n', help='to clear the nextHopIpAddress', default=None, action="store_true")
        optional.add_argument('-s', help='to clear the subscriptionId', default=None, action="store_true")
        optional.add_argument('-g', help='to clear the resourceGroup', default=None, action="store_true")
        optional.add_argument('-t', help='to clear the routeTableName', default=None, action="store_true")
        optional.add_argument('-a', help='to clear the applicationId', default=None, action="store_true")
        optional.add_argument('-d', help='to clear the tenantId', default=None, action="store_true")
        optional.add_argument('-k', help='to clear the applicationKey', default=None, action="store_true")
        optional.add_argument('-r', help='to clear the route', default=None, action="store_true")



    def create_node_parser(self, optional, required):

        required.add_argument('-p', help='<cloud_provider>   {azure | azusgov | azchina}', choices=['azure', 'azusgov', 'azchina'],
                              default=None, required=True)
        required.add_argument('-n', help='<nextHopIpAddress>', default=None, required=True)
        required.add_argument('-t', help='<routeTableName>', default=None, required=True)
        optional.add_argument('-s', help='<subscriptionId>', default=None)
        optional.add_argument('-g', help='<resourceGroup>', default=None)
        optional.add_argument('-r', help='<route>   e.g. 15.0.0.0/8', default=None)
        optional.add_argument('-a', help='to add the applicationId', default=None)
        optional.add_argument('-d', help='to add the tenantId', default=None)
        optional.add_argument('-k', help='to add the applicationKey', default=None)

    def validate_inputs(self, args):
        response = {}
        resp_msg = '''Please make sure you have MSI (Managed service identity) feature available 
in this region of Azure and it is configured to allow this CSR VM an contributer/network contributor 
access to the route table to be able to change the routes. At last, please do 
'node_event.py -i {} -e verify' to verify node is configured correctly.\n
            '''.format(str(args.i))
        response['code'] = 200
        response['msg'] = ""

        #Returning if Subscription ID is missing
        if not args.s or not args.g:
            response['code'] = 199
            response['msg'] = "Unable to validate nodes. Please verify node configuration."
            if not args.s:
                response['msg'] = response['msg'] + "\n199: subscriptionId is not found in input parameters. Please configure valid subscriptionId using set_params.py client_api"
            if not args.g:
                response['msg'] = response['msg'] + "\n199: resourceGroup is not found in input parameters. Please configure valid resourceGroup using set_params.py client_api"
            return json.dumps(response)

        if (not args.a or not args.s or not args.d or not args.k) and args.p != 'azure':
            log.warning(resp_msg)
            response['code'] = 199
            response['msg'] = resp_msg

        node = self.create_node_from_args(args)

        foundRoute = False
        routetable = self.get_route_table(node)
        if routetable is not None:
            # Walk through all the routes in the current route table
            if args.r:
                for i, route in enumerate(routetable["properties"]["routes"]):
                    if route['properties']['nextHopType'] == 'VirtualAppliance':
                        # Updating a single route in the table.  Need to find the right one.
                        if node['route'] == route['properties']['addressPrefix']:
                                foundRoute = True
                if foundRoute:
                    response['msg'] += "Verified route {} is present in the routetable".format(node['route'])
                else:
                    response['code'] = 199
                    response['msg'] += "Route {} is not found in the routetable".format(node['route'])

            # checking if the node configured is for all routes
            else:
                response['code'] = 200
                response['msg'] += "Verified node configuration can fetch all routes in routetable"

        else:
            response['code'] = 199
            response['msg'] = "Unable to validate nodes, please verify the node configuration"
        return json.dumps(response)

    def set_node_parser(self,  optional, required):

        optional.add_argument('-s', help='to set the subscriptionId', default=None)
        optional.add_argument('-r', help='to set the route', default=None)
        optional.add_argument('-g', help='to set the resourceGroup', default=None)
        optional.add_argument('-t', help='to set the routeTableName', default=None)
        optional.add_argument('-n', help='to set the nextHopIpAddress', default=None)
        optional.add_argument('-a', help='to set the applicationId', default=None)
        optional.add_argument('-d', help='to set the tenantId', default=None)
        optional.add_argument('-k', help='to set the applicationKey', default=None)


    def validate_nexthop(self, nextHop):
        '''
        This method is used to validate if the nextHop is in the right IP format
        :param nextHop:
        :return: Returns 0 or 1 based on the nextHop
        '''
        try:
            ipaddress.ip_address(unicode(nextHop))
            return 0
        except Exception as e:
            return 1

    # def validate_string(self, resourceInput):
    #     '''
    #     Validating the ResourceGroup naming conventions
    #     :param resourceInput:
    #     :return:
    #     '''
    #     if re.match(r"^[\w\._\(\)]+$", resourceInput):
    #         log.info("Validated the input: {}".format(resourceInput))
    #         return resourceInput
    #     else:
    #         raise argparse.ArgumentTypeError("Not in a alphanumeric, underscore, parentheses, hyphen, period (except at end), and Unicode characters format")

    def create_node_from_args(self, args):
        node = {}
        if args.p:
            node['cloud'] = args.p
        if args.s:
            node['subscriptionId'] = args.s
        if args.n:
            node['nextHop'] = args.n
        if args.g:
            node['resourceGroup'] = args.g
        if args.t:
            node['routeTableName'] = args.t
        if args.r:
            node['route'] = args.r
        if args.a:
            node['appId'] = args.a
        if args.d:
            node['tenantId'] = args.d
        if args.k:
            node['appKey'] = args.k

        return node


    def validate_set_param_inputs(self, node, new_params):
        response = {}
        resp_msg = '''Please make sure you have MSI (Managed service identity) feature available 
        in this region of Azure and it is configured to allow this CSR VM an contributer/network contributor 
        access to the route table to be able to change the routes. At last, please do 
        'node_event.py -i {} -e verify' to verify node is configured correctly.\n
                    '''.format(str(node['index']))
        response['code'] = 200
        response['msg'] = ""
        if not node.has_key("resourceGroup") or not node.has_key('subscriptionId'):
            response['code'] = 199
            response['msg'] = "Unable to validate nodes. Please verify node configuration"
            if not node.has_key("resourceGroup"):
                response['msg'] =  response['msg'] + "\n199: resourceGroup not found in input parameters"
            if not node.has_key("subscriptionId"):
                response['msg'] = response['msg'] + "\n199: subscriptionId not found in input parameters"
            return response

        if (not node.has_key('appId') or not node.has_key('subscriptionId') or not node.has_key('tenantId') or not node.has_key('appKey')) and node.has_key('cloud') != 'azure':
            log.warning(resp_msg)
            response['code'] = 199
            response['msg'] = resp_msg

        foundRoute = False
        routetable = self.get_route_table(node)
        if routetable is not None:
            # Walk through all the routes in the current route table
            if node.has_key("route"):
                for i, route in enumerate(routetable["properties"]["routes"]):
                    if route['properties']['nextHopType'] == 'VirtualAppliance':
                        # Updating a single route in the table.  Need to find the right one.
                        if node['route'] == route['properties']['addressPrefix']:
                            foundRoute = True
                if foundRoute:
                    response['msg'] += "Verified route {} is present in the routetable".format(node['route'])
                else:
                    response['code'] = 199
                    response['msg'] += "Route {} is not found in the routetable".format(node['route'])

            # checking if the node configured is for all routes
            else:
                response['code'] = 200
                response['msg'] += "Verified node configuration can fetch all routes in routetable"

        else:
            response['code'] = 199
            response['msg'] = "Unable to validate nodes, please verify the node configuration"
        return response

    # Taking cloud's input if a revert event is needed if none of nodes are configured as Primary
    def do_revert(self):
        return False



from distutils.core import setup
from setuptools.command.install import install
from subprocess import check_call
import os, platform


project_name = 'csr_azure_utils'
project_ver = '1.1.5'

'''
=======================================================================================
Note
=======================================================================================
This file is crucial to installation of csr_azure_utils. 
Before committing any changes to this file, please test installation of csr_azure_utils
beforehand on your local machine/MacBook and in Guestshell running in CSR.
'''

class PostInstallCommand(install):
    """Post-installation for installation mode."""

    def run(self):
        try:
            print "We are running in the postInstallCommand"
            if "centos" in platform.dist()[0].lower() and  "guestshell" in os.popen("whoami").read().strip():
                cwd = os.path.dirname(os.path.realpath(__file__))
                check_call("bash %s/install.sh" % cwd, shell=True)
                check_call("sudo cp auth-token.service /etc/systemd/user/",
                           shell=True)
                check_call("sudo systemctl enable /etc/systemd/user/auth-token.service",
                           shell=True)
            else:
                print "Skipping auth-service setup, csr_azure_utils couldn't find either guestshell as \
                active user or platform not centos"
            install.run(self)
        except Exception as e:
            print "Unable to setup the token service via systemd"

setup(
    name=project_name,
    packages=["csr_cloud"],
    version=project_ver,
    description='Utilities for csr1000v on Azure',
    author='Christopher Reder',
    author_email='creder@cisco.com',
    scripts=['csr_cloud/clear_aad_application_list.py',
             'csr_cloud/clear_default_aad_app.py',
             'csr_cloud/clear_token.py',
             'csr_cloud/refresh_token.py',
             'csr_cloud/set_default_aad_app.py',
             'csr_cloud/show_auth_applications.py'
            ],
    # use the URL to the github repo
    url='https://github4-chn.cisco.com/csr1000v-azure/' + project_name,
    download_url='https://github4-chn.cisco.com/csr1000v-azure/' + project_name + '/archive/' + \
         project_ver + '.tar.gz',
    keywords=['cisco', 'azure', 'guestshell', 'csr1000v'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],
    license="MIT",
    include_package_data=True,
    install_requires=[
        'python-crontab',
        'pathlib',
        'azure-storage',
        'configparser',
        'pyopenssl',
        'msrest',
        'msrestazure',
        'paramiko'
    ],
    cmdclass={
        'install': PostInstallCommand,
    }
)




#!/usr/bin/env python

import os
from os.path import expanduser
import requests
import subprocess
import traceback
import logging
from logging.handlers import RotatingFileHandler
from datetime import datetime, timedelta

# Specify files accessed by this script in guestshell
base_dir = expanduser('~') + '/cloud/authMgr/'
get_response_file = base_dir + "token_get_rsp"
debug_file = base_dir + "token_svr.log"
cert_file = "/etc/ssl/certs/ca-bundle.trust.crt"
token_file = base_dir + "token_file"
msi_dir = "/var/log/azure/Microsoft.ManagedIdentity.ManagedIdentityExtensionForLinux"
msi_service_file = "/etc/systemd/system/azuremsixtn.service"



class MSIAuth:

    def __doc__(self):
        """This class represents a Managed Service Identity (MSI) application in Azure Active Directory"""

    def __init__(self, logger): #pragma: no cover
        self.token = ''
        self.token_expiry_time = datetime.utcnow() - timedelta(minutes=1)
        self.logger = logger

        if not os.path.exists(base_dir):
            os.makedirs(base_dir)

        # Try to get a token
        token = self.get_token()
        if token == '':
            self.logger.error("MSI app failed verification: unable to obtain a token")

    def msi_get_token_by_http(self):
        # Specify the HTTP GET request to obtain a token
        url = "http://169.254.169.254/metadata/identity/oauth2/token?api-version=2018-02-01"
        payload = {'resource': 'https://management.azure.com/'}
        header = {'Metadata': 'true'}

        # Send the HTTP GET request for the token
        try:
            response = requests.get(url, params=payload, verify=False, headers=header)
        except requests.exceptions.RequestException as e:
            self.logger.exception("Server: msi_get_token: request had an error %s", e)
            return ''

        if 200 != response.status_code:
            self.logger.error("Server: msi_get_token: request failed rc=%d", response.status_code)
            with open(token_file, 'wb') as token_fh:
                for chunk in response.iter_content(chunk_size=64):
                    token_fh.write(chunk)
            return ''

        # Parse the HTTP GET response
        try:
            token = response.json()["access_token"]
            self.logger.info("Server: msi_get_token: obtained token by HTTP")
        except Exception as e:
            self.logger.exception("Server: msi_get_token: caught exception %s", e)
            tb = traceback.format_exc()
            self.logger.exception("%s", tb)
            token = ''

        return token


    def msi_get_token_by_extension(self):
        # Check if user has successfully installed the Managed Identity Extension
        if not os.path.exists(msi_dir):
            # Directory does not exist.  Don't use MSI
            self.logger.info("Server: msi_get_token: MSI not installed")
            return ''

        # Check if the MSI service is running
        try:
            subprocess.check_output("pgrep -f msi-extension", shell=True)
        except:
            # Check if the Managed Identity Extension has been installed
            if os.path.exists(msi_service_file):
                # File has been installed.  Try to start the service.
                os.system("sudo systemctl enable azuremsixtn")
                os.system("sudo systemctl start azuremsixtn")
                self.logger.exception("Server: msi_get_token: started MSI service")
            else:
                # Directory does not exist.  Don't use MSI
                self.logger.exception("Server: msi_get_token: MSI not installed")
                return ''

        # Specify the HTTP GET request to obtain a token
        url = "http://localhost:50342/oauth2/token"
        payload = {'resource': 'https://management.azure.com/'}
        header = {'Metadata': 'true'}

        # Send the HTTP GET request for the token
        response = requests.get(url, params=payload, verify=False, headers=header)

        if 200 != response.status_code:
            self.logger.error("Server: msi_get_token: request failed rc=%d", response.status_code)
            with open(token_file, 'w') as token_fh:
                token_fh.write(response.json())
            return ''

        # Parse the HTTP GET response
        token = response.json()["access_token"]
        self.logger.info("Server: msi_get_token: obtained token by extension")

        return token

    def get_token(self):
        if datetime.utcnow() < self.token_expiry_time:
            # Cached token is still valid. Just return it.
            return self.token
        else:
            # Need to refresh the token
            token = self.msi_get_token_by_http()
            if token != '':
                self.token = token
                self.token_expiry_time = datetime.utcnow() + timedelta(minutes=5)
                return self.token
            else:
                # Try getting token by MSI extension
                token = self.msi_get_token_by_extension()
                if token != '':
                    self.token = token
                    self.token_expiry_time = datetime.utcnow() + timedelta(minutes=5)
                    return self.token
                return ''

    def refresh_token(self):
        # Invalidate the current token
        self.token = ''
        self.token_expiry_time = datetime.utcnow() - timedelta(minutes=1)
        self.logger.info("Token successfully cleared")
        # Return a new token
        return self.get_token()

    def clear_token(self):
        self.token = ''
        self.token_expiry_time = datetime.utcnow()
        self.logger.info("Token successfully cleared")
        return 'OK'

    def show(self):
        buf = ''
        if self.token != '':
            buf = buf + "\nToken:\t%s" % self.token
            buf = buf + "\nExpires:\t%s" % self.token_expiry_time
            if self.token_expiry_time > datetime.utcnow():
                buf = buf + "\nToken is valid"
            else:
                buf = buf + "\nToken is expired"
        else:
            buf = buf + "\nNo token found"
        return buf



#!/bin/env python
'''
Cisco Copyright 2018
Author: Vishlesh Patel <vishlpa4@cisco.com>

FILENAME: EVENTHUB_UTILS.PY
'''

from azure.servicebus.servicebusservice import ServiceBusService
import os
import logging
from msrestazure.azure_exceptions import CloudError
import azure.mgmt.eventhub.models
from azure.mgmt.eventhub.models import EHNamespace, Sku, SkuName, AccessRights, Eventhub, CaptureDescription
from azure.mgmt.eventhub.models import ErrorResponseException, ErrorResponse, AuthorizationRule, AccessKeys, EncodingCaptureDescription, Destination
from azure.common.credentials import ServicePrincipalCredentials
from azure.mgmt.eventhub import EventHubManagementClient
import configparser

class EventHubUtils():
    def __init__(self, feature=None):
        self.feature = feature if feature is not None else __name__
        self.logger = logging.getLogger(self.feature)
        self.region = os.getenv('DEFAULT_REGION', "eastus")
        self.app_id = os.getenv("AZURE_CLIENT_ID", "")
        self.app_key = os.getenv("AZURE_CLIENT_SECRET", "")
        self.subscription_id = os.getenv("SUBSCRIPTION_ID", "")
        self.tenant = os.getenv("TENANT_ID", "")
        self.customer_resource_group = os.getenv("CUSTOMER_RESOURCE_GROUP","")
        self.eventhub_client = self.get_event_hub_client()
        self.eventhub_rule_name = os.getenv("EVENTHUB_PUBLISHER_AUTH_RULE_NAME", "publisherRule")
        self.eventhub_rule_key = os.getenv("EVENTHUB_PUBLISHER_AUTH_RULE_KEY", "zReduFf8ImU33lfaofcfb3DGjkzd+sHgIvuhlLKgi2o=")

    def load_env(self):
        try:
            config = configparser.ConfigParser()
            config.optionxform = str
            home = os.path.expanduser("~")
            filename = home + '/.azure/credentials'
            if os.path.exists(filename):
                config.read(filename)
                for key in config['default']:
                    value = config.get('default', key)
                    if value.startswith('"') and value.endswith('"'):
                        value = value[1:-1]
                    os.environ[key] = str(value)
        except Exception as e:
            self.logger.exception(e)

    def get_event_hub_client(self):
        try:
            self.credentials = ServicePrincipalCredentials(
                client_id=self.app_id,
                secret=self.app_key,
                tenant=self.tenant
            )
            eventhub_client = EventHubManagementClient(self.credentials, self.subscription_id)
            return eventhub_client
        except Exception as e:
            self.logger.exception(e)

    def create_namespace(self, namespace_name, rg=""):
        '''

        :return: creates an event hub resource and returns namespace object
        '''
        try:
            # Create a Namespace
            rg = self.customer_resource_group if rg == "" else rg
            namespaceparameter=EHNamespace(location=self.region,sku=Sku(name=SkuName.standard))
            poller = self.eventhub_client.namespaces.create_or_update(rg, namespace_name, namespaceparameter)
            creatednamespace = poller.result()
            self.logger.info("created a namespace - {} successfully.".format(namespace_name))
            return True
        except Exception as e:
            self.logger.exception(e)
            return False

    def create_authorization_rule(self,  namespace_name, rule_name, role="Publisher", rg=""):
        '''

        :param rg:
        :param namespace_name:
        :param rule_name: name or rule
        :param role: datatype - string.
                Publisher - can only send/publish event
                Listener - can only listen to events
                Admin - Can do both publish and listen
        :return:
                returns all keys for the rule.
                Accessible by using returned object.primary_key,
                                    returned object.secondary_key
                                    returned object.secondary_connection_string
                                    returned object.primary_connection_string
        '''
        try:
            # Create a new authorization rule
            authoRule_name = rule_name
            rg = self.customer_resource_group if rg == "" else rg
            role = str(role)
            accessRights = []
            if role.lower() == 'publisher' or role.lower() == 'admin':
                accessRights.append(AccessRights('Send'))
            if role.lower() == 'listener' or role.lower() == 'admin':
                accessRights.append(AccessRights('Listen'))
            createnamespaceauthorule = self.eventhub_client.namespaces.create_or_update_authorization_rule(rg,
                                       namespace_name, authoRule_name,
                                       accessRights)
            # List keys for the authorization rule
            listkeysauthorizationrule = self.eventhub_client.namespaces.list_keys(rg, namespace_name,
                                                                                  authoRule_name)
            return listkeysauthorizationrule
        except Exception as e:
            self.logger.exception(e)
            return None

    def create_event_hub_with_capture_feature(self, eventhub_name, rg, namespace, sa_resourece_id):
        try:
            # Create a Eventhub
            eventhubparameter = Eventhub(
                message_retention_in_days=4,
                partition_count=4,
                capture_description=CaptureDescription(
                    enabled=True,
                    encoding=EncodingCaptureDescription.avro,
                    interval_in_seconds=120,
                    size_limit_in_bytes=10485763,
                    destination=Destination(
                        name="EventHubArchive.AzureBlockBlob",
                        storage_account_resource_id=sa_resourece_id,
                        blob_container="container",
                        archive_name_format="{Namespace}/{EventHub}/{PartitionId}/{Year}-{Month}-{Day}-{Hour}-{Minute}-{Second}")
                )
            )
            createdeventhubresponse = self.eventhub_client.event_hubs.create_or_update(rg,
                                                                                       namespace, eventhub_name,
                                                                                       eventhubparameter)
            return True
        except Exception as e:
            self.logger.exception(e)
            return False

    def create_event_hub(self, eventhub_name, rg, namespace):
        try:
            # Create a Eventhub
            eventhubparameter = Eventhub(
                message_retention_in_days=4,
                partition_count=4
                )
            createdeventhubresponse = self.eventhub_client.event_hubs.create_or_update(rg,
                                                                                       namespace, eventhub_name,
                                                                                       eventhubparameter)
            return True
        except Exception as e:
            self.logger.exception(e)
            return False

    def delete_event_hub(self, eventhub_name, rg, namespace):
        try:
            geteventhubresponse = self.eventhub_client.event_hubs.delete(rg, namespace,
                                                                         eventhub_name)
            return True
        except Exception as e:
            self.logger.exception(e)
            return False

    def delete_namespace(self, rg, namespace):
        try:
            deletenamespace = self.eventhub_client.namespaces.delete(rg, namespace).result()
            return True
        except Exception as e:
            self.logger.exception(e)
            return False

    def send_notification(
            self,
            topicarn,
            msg="",
            subject="",
            msgtype='string',
            message_attributes={}):
        '''
            :param topicarn: input format - "eventhub_namespace_name/eventhub_name"
            :param msg:
            :param subject:
            :param msgtype:
            :param message_attributes:
            :return:
        '''
        try:
            namespace, eh_name = topicarn.split("/")
            sbs = ServiceBusService(namespace, shared_access_key_name=self.eventhub_rule_name,
                                    shared_access_key_value=self.eventhub_rule_key)
            sbs.send_event(eh_name, message_attributes)
            return True
        except Exception as e:
            self.logger.exception("topicarn: {}, exception: {}".format(topicarn, e))
            return False

if __name__ == "__main__":
    foo = EventHubUtils()
    foo.create_namespace("testnamespace1jun17", "rgeventhubjun17")
    keys = foo.create_authorization_rule("testnamespace1jun17", "listenerRole", "listener", "rgeventhubjun17")
    print(keys.primary_connection_string)
    sa_id = "/subscriptions/b7fd700c-0b05-44ba-823a-6e954c480149/resourcegroups/rgeventhubjun17/providers/Microsoft.Storage/storageAccounts/testeventhubjul17"
    foo.create_event_hub_with_capture_feature("eventhub1", "rgeventhubjun17", "testnamespace1jun17", sa_id)
    msg = str({ "event": "scale_out", "pip" : "10.x.x.x" })
    foo.send_notification("testnamespace1jun17/eventhub1", message_attributes=msg)
    #foo.create_event_hub("eventhub2", "rgeventhubjun17", "testnamespace1jun17")
